package com.xyzc.start.service.impl;

import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.constant.SchemeConst;
import com.xyzc.start.domain.Scheme;
import com.xyzc.start.domain.bo.CreateOrderBo;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.*;
import com.xyzc.start.service.ISchemeService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
@RequiredArgsConstructor
public class SchemeServiceImpl implements ISchemeService {

    private final SchemeMapper schemeMapper;
    private final MemberCardMapper memberCardMapper;
    private final EquityMapper equityMapper;
    private final AiModelMapper aiModelMapper;

    private final FuelPackageMapper fuelPackageMapper;

    /**
     * 获取当前方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentMember() {
        SchemeVo scheme = schemeMapper.getCurrentMember();

        // 有且只有一个会员卡套餐生效
        // 根据套餐查询会员卡
        // TODO 代理商id暂时写死
        List<MemberCardVo> memberCardVos = memberCardMapper.queryVoListBySchemeId(scheme.getId(), 1810310843224322049L);
        // 让memberCardVos中根据memberId合并成新的List
        List<List<MemberCardVo>> memberList = new ArrayList<>();
        List<MemberCardVo> collect = new ArrayList<>();
        for (MemberCardVo memberCardVo : memberCardVos) {
            // collect不为空并且memberCardId不一致时新建一个list
            if (!collect.isEmpty() && !collect.get(0).getMemberCardId().equals(memberCardVo.getMemberCardId())) {
                memberList.add(collect);
                collect = new ArrayList<>();
            }
            // 对当前值进行处理
            memberCardVo.setSaveMoney(memberCardVo.getPrice().subtract(memberCardVo.getStrikethroughPrice()));
            collect.add(memberCardVo);
            memberCardVo.setMemberEquityList(grouth());

        }
        memberList.add(collect);

        // 将会员卡列表绑定到方案中
        scheme.setMemberCardList(memberList);

        return scheme;
    }

    private static List<MemberEquityVo> grouth() {
        ArrayList<MemberEquityVo> objects = new ArrayList<>();

        MemberEquityVo memberCardVo = new MemberEquityVo();
        memberCardVo.setMemberEquityid(1);
        memberCardVo.setMemberEquityName("应用权益");
        memberCardVo.setMemberEquityCount(230);

        MemberEquityVo memberCardVo2 = new MemberEquityVo();
        memberCardVo2.setMemberEquityid(2);
        memberCardVo2.setMemberEquityName("AI绘画");
        memberCardVo2.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo3 = new MemberEquityVo();
        memberCardVo3.setMemberEquityid(2);
        memberCardVo3.setMemberEquityName("AI绘画");
        memberCardVo3.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo4 = new MemberEquityVo();
        memberCardVo4.setMemberEquityid(2);
        memberCardVo4.setMemberEquityName("AI绘画");
        memberCardVo4.setMemberEquityCount(2320);

        MemberEquityVo memberCardVo5 = new MemberEquityVo();
        memberCardVo5.setMemberEquityid(2);
        memberCardVo5.setMemberEquityName("AI绘画");
        memberCardVo5.setMemberEquityCount(2320);

        return objects;
    }

    /**
     * 根据id获取燃料信息
     *
     * @param fuelId
     * @return
     */
    @Override
    public List<FuelPackageVo> queryCurrentFuelById(Long fuelId) {

        return schemeMapper.queryFuelPackageById(fuelId);
    }

    /**
     * 分页查询燃料规则详情
     *
     * @param modelId 模型id
     * @return
     */
    @Override
    public List<AiModelLawVo> queryModelLawList(Long modelId) {

        return aiModelMapper.queryModelLawList(modelId);
    }

    /**
     * 根据会员卡id获取会员卡信息
     *
     * @param memberId
     * @return
     */
    @Override
    public MemberCardVo queryCurrentMemberById(Long memberId) {

        // 传入用户自己的值去
        return schemeMapper.queryMemberCardByMemberId(SecurityUser.getUserId(), memberId);
    }

    /**
     * 获取当前生效的燃料方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentFuel() {
        List<SchemeVo> currentFuel = schemeMapper.getCurrentFuel();
        // 有且只有一个会员卡套餐生效
        AssertUtils.isUnique(currentFuel, ErrorCode.SCHEME_NOT_UNIQUE);
        SchemeVo scheme = currentFuel.get(0);

        // 根据套餐查询燃料
        List<FuelPackageVo> fuelPackageVo = schemeMapper.queryFuelPackageById(scheme.getId());

        scheme.setFuelPackageList(fuelPackageVo);

        return scheme;
    }

    /**
     * 创建订单
     *
     * @param bo
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseUserOrderVo createOrder(CreateOrderBo bo) {
        List<MemberCardOrderVo> memberCardOrderList = null;
        // 或者燃料包
        List<FuelPackageOrderVo> fuelPackageOrderList = null;

        if (SchemeConst.MEMEBER_TYPE.equals(bo.getType())) {
            Scheme scheme = schemeMapper.querySchemeByMemberId(bo);
            // 没有生效的套餐抛出异常
            AssertUtils.notNull(scheme, ErrorCode.SCHEME_NOT_EXIST);

            memberCardOrderList = memberCardMapper.queryOrderByMemberId(bo, 1810310843224322049L);

        } else if (SchemeConst.FUEL_TYPE.equals(bo.getType())) {
            Scheme scheme = schemeMapper.querySchemeByFuel(bo);
            // 没有生效的套餐抛出异常
            AssertUtils.notNull(scheme, ErrorCode.SCHEME_NOT_EXIST);

            fuelPackageOrderList = fuelPackageMapper.queryOrderByFuelPackageId(bo, 1810310843224322049L);
        }

        // TODO 生成一个假数据
        BaseUserOrderVo baseUserOrderVo = new BaseUserOrderVo();
        baseUserOrderVo.setMemberCardList(memberCardOrderList);
        baseUserOrderVo.setFuelPackageList(fuelPackageOrderList);

        return baseUserOrderVo;
    }
}
